"""
Let's create a Mammoth object in the root folder so we can access
views through the publisher:

  >>> from zope.app.testing.functional import getRootFolder
  >>> root = getRootFolder()
  >>> root['manfred'] = Mammoth()

As an anonymous user, we only see the unprotected menu items:

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser('http://localhost/manfred/showmenu')
  >>> print browser.contents
  [{'action': '',
    'description': '',
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': [{'action': 'optionone',
                 'description': u'',
                 'extra': None,
                 'icon': None,
                 'selected': u'',
                 'submenu': None,
                 'title': 'Option one'}],
    'title': 'Options'},
   {'action': 'edit',
    'description': u'',
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': None,
    'title': 'Edit'},
   {'action': 'index',
    'description': u'',
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': None,
    'title': 'View'}]

After logging in as a manager, we also see the protected one:

  >>> browser.addHeader('Authorization', 'Basic mgr:mgrpw')
  >>> browser.open('http://localhost/manfred/showmenu')
  >>> print browser.contents
  [{'action': '',
    'description': '',
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': [{'action': 'optionone',
                 'description': u'',
                 'extra': None,
                 'icon': None,
                 'selected': u'',
                 'submenu': None,
                 'title': 'Option one'}],
    'title': 'Options'},
   {'action': '',
    'description': '',
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': [{'action': 'configoption',
                 'description': u'',
                 'extra': None,
                 'icon': None,
                 'selected': u'',
                 'submenu': None,
                 'title': 'Protected configuration'}],
    'title': 'Setup'},
   {'action': 'edit',
    'description': u'',
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': None,
    'title': 'Edit'},
   {'action': 'index',
    'description': u'',
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': None,
    'title': 'View'},
   {'action': 'manage',
    'description': u'',
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': None,
    'title': 'Manage'}]

"""
from grokcore.component import Context, name, title, description
from grokcore.view import View
from grokcore.security import Permission, require

import megrok.menu

from pprint import pformat
from zope.component import getUtility
from zope.app.publisher.interfaces.browser import IBrowserMenu

class Mammoth(Context):
    pass

class Actions(megrok.menu.Menu):
    name('actions')
    title('Actions')
    description('')

# You can either refer to the menu class itself:

class Index(View):
    title('View')
    megrok.menu.menuitem(Actions)

    def render(self):
        return 'index'

# or you can refer to its identifier:

class Edit(View):
    title('Edit')
    megrok.menu.menuitem('actions')

    def render(self):
        return 'edit'

# also you can define sub-menus items
class Options(megrok.menu.SubMenuItem):
    name('options')
    title('Options')
    description('')

    megrok.menu.menuitem('actions')

class OptionOne(View):
    title('Option one')
    megrok.menu.menuitem('options')

    def render(self):
        return 'option one'

# Here's a view that's protected by a permission. We expect the menu
# item that we configure for it to have the same permission setting:

class ManageStuff(Permission):
    name('my.ManageStuff')

class Manage(View):
    require(ManageStuff)
    title('Manage')
    megrok.menu.menuitem('actions')

    def render(self):
        return 'manage'

#Sub menus item are also available to be protected using a permission
class Setup(megrok.menu.SubMenuItem):
    require(ManageStuff)
    name('setup')
    title('Setup')
    description('')

    megrok.menu.menuitem('actions')

class ConfigOption(View):
    title('Protected configuration')
    megrok.menu.menuitem('setup')

    def render(self):
        return 'Configuration'

class ShowMenu(View):

    def render(self):
        menu = getUtility(IBrowserMenu, 'actions')
        return pformat(menu.getMenuItems(self.context, self.request))

def test_suite():
    from zope.testing import doctest
    from megrok.menu.tests import FunctionalLayer
    suite = doctest.DocTestSuite()
    suite.layer = FunctionalLayer
    return suite



"""
  >>> from zope.component import getUtility
  >>> from zope.app.publisher.interfaces.browser import IBrowserMenu
  >>> from zope.publisher.browser import TestRequest

A menu is available as a named utility providing ``IBrowserMenu``.

  >>> menu = getUtility(IBrowserMenu, 'ordered')
  >>> manfred = Mammoth()
  >>> request = TestRequest()

In order to retrieve the menu items, we need to pass in a context
object and a request.  The menu then determines which menu items are
available for this particular object and the principal that's attached
to the request:

  >>> from pprint import pprint
  >>> pprint(menu.getMenuItems(manfred, request))
  [{'action': 'cluelessview',
    'description': "We keep the 'menuitem' directive order info if present !",
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': None,
    'title': 'What should I chose ?'},
   {'action': 'firstview',
    'description': u'',
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': None,
    'title': 'my first view'},
   {'action': 'secondview',
    'description': u'',
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': None,
    'title': 'my second view'},
   {'action': 'thirdview',
    'description': u'',
    'extra': None,
    'icon': None,
    'selected': u'',
    'submenu': None,
    'title': 'my third view'}]

"""

import megrok.menu
import grokcore.viewlet as grok


class Mammoth(grok.Context):
    pass


class OrderedMenu(megrok.menu.Menu):
    grok.name('ordered')
    grok.title('My menu')


class SecondView(grok.View):
    grok.order(2)
    grok.title('my second view')
    megrok.menu.menuitem('ordered')

    def render(self):
        return 'test'


class ThirdView(grok.View):
    grok.title('my third view')
    megrok.menu.menuitem('ordered', order=3)

    def render(self):
        return 'test'


class FirstView(grok.View):
    grok.order(1)
    grok.title('my first view')
    megrok.menu.menuitem('ordered')

    def render(self):
        return 'test'


class CluelessView(grok.View):
    grok.order(5)
    grok.title('What should I chose ?')
    grok.description("We keep the 'menuitem' directive "
                     "order info if present !")
    megrok.menu.menuitem('ordered', order=0)

    def render(self):
        return 'test'
    

def test_suite():
    from zope.testing import doctest
    from megrok.menu.tests import FunctionalLayer
    suite = doctest.DocTestSuite()
    suite.layer = FunctionalLayer
    return suite



from setuptools import setup, find_packages

long_description = (open("README.txt").read()
                    + '\n\n' +
                    open("CHANGES.txt").read())

setup(name='megrok.menu',
      version='0.3',
      description="Grok extension to configure browser menus",
      long_description=long_description,
      classifiers=['Programming Language :: Python',
                   'Environment :: Web Environment',
                   'Framework :: Zope3',
                   'License :: OSI Approved :: Zope Public License',
                   ],
      keywords='',
      author='Philipp von Weitershausen',
      author_email='philipp@weitershausen.de',
      url='http://pypi.python.org/pypi/megrok.menu',
      license='ZPL',
      packages=find_packages('src'),
      package_dir = {'': 'src'},
      namespace_packages=['megrok'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'grokcore.component',
          'grokcore.view',
          'grokcore.viewlet',
          'grokcore.security',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



import martian
from martian.error import GrokError

import grokcore.view
import grokcore.viewlet
import grokcore.security
import grokcore.component

from grokcore.security.util import protect_getattr
from grokcore.view.meta.views import ViewSecurityGrokker, default_view_name

from zope.configuration.exceptions import ConfigurationError
from zope.app.publisher.browser.menumeta import menuDirective, \
     menuItemDirective, subMenuItemDirective

from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.publisher.interfaces.browser import IBrowserPage

import megrok.menu


class MenuGrokker(martian.ClassGrokker):
    martian.component(megrok.menu.Menu)
    martian.priority(1500)
    martian.directive(grokcore.component.name, get_default=default_view_name)
    martian.directive(grokcore.component.title, default=u'')
    martian.directive(grokcore.component.description, default=u'')

    def execute(self, factory, config, name, title, description, **kw):
        menuDirective(config, id=name, class_=factory,
                      title=title, description=description)
        return True


class SubMenuItemGrokker(martian.ClassGrokker):
    martian.component(megrok.menu.SubMenuItem)

    # We want to do this after MenuGrokker.
    martian.priority(1000)

    martian.directive(grokcore.component.name, get_default=default_view_name)
    martian.directive(grokcore.component.title, default=u'')
    martian.directive(grokcore.component.description, default=u'')
    martian.directive(grokcore.viewlet.order)
    martian.directive(grokcore.component.context)
    martian.directive(grokcore.view.layer, default=IDefaultBrowserLayer)
    martian.directive(grokcore.security.require, name='permission')

    martian.directive(megrok.menu.menuitem)

    def execute(self, factory, config, name, title, description,
                order = None, menuitem=None, context=None,
                layer=None, permission=None):

        menuDirective(config, id=name, class_=factory,
                      title=title, description=description)
        
        if menuitem is None:
            return False

        menu_id, icon, filter, enforced_order, extra = menuitem
       
        if enforced_order is None:
            enforced_order = order[0] or 0

        try:
            menu = config.resolve('zope.app.menus.'+menu_id)
        except ConfigurationError, v:
            raise GrokError("The %r menu could not be found.  Please use "
                            "megrok.menu.Menu to register a menu first."
                            % menu_id, factory)

        subMenuItemDirective(
            config, menu=menu, for_=context, submenu=name,
            title=title, description=description, icon=icon,
            filter=filter, permission=permission, layer=layer,
            order=enforced_order, action='', extra=extra
            )

        for method_name in IBrowserPage:
            if method_name == '__call__':
                continue
            config.action(
                discriminator=('protectMenuName', factory, method_name),
                callable=protect_getattr,
                args=(factory, method_name, permission),
                )
        return True


class MenuItemGrokker(ViewSecurityGrokker):
    martian.directive(megrok.menu.menuitem)
    martian.directive(grokcore.component.context)
    martian.directive(grokcore.viewlet.order)
    martian.directive(grokcore.view.layer, default=IDefaultBrowserLayer)
    martian.directive(grokcore.component.name, get_default=default_view_name)
    martian.directive(grokcore.component.title, default=u'')
    martian.directive(grokcore.component.description, default=u'')

    def execute(self, factory, config, permission, order, context=None,
                layer=None, name=u'', menuitem=None, description=u'',
                title=u''):

        if menuitem is None:
            return False
        
        menu_id, icon, filter, enforced_order, extra = menuitem
        
        if enforced_order is None:
            enforced_order = order[0] or 0

        try:
            menu = config.resolve('zope.app.menus.'+menu_id)
        except ConfigurationError, v:
            raise GrokError("The %r menu could not be found.  Please use "
                            "megrok.menu.Menu to register a menu first."
                            % menu_id, factory)
        menuItemDirective(config, menu=menu, for_=context, action=name,
                          title=title, description=description, icon=icon,
                          filter=filter, permission=permission, layer=layer,
                          order=enforced_order, extra=extra)

        # Menu items check whether the view that they refer to can be
        # traversed to.  Unfortunately, views will end up being
        # security proxied during that fake traversal.  For this to
        # work then, we must define a checker not only for __call__
        # but also for browserDefault and those other methods from
        # IBrowserPage:
        ######
        # This is no longer needed. protect_getattr function will take care.
        #if permission is None:
        #    permission = 'zope.Public'

        for method_name in IBrowserPage:
            if method_name == '__call__':
                continue
            config.action(
                discriminator=('protectMenuName', factory, method_name),
                callable=protect_getattr,
                args=(factory, method_name, permission),
                )

        return True
